@@ -57,9 +57,9 @@ NumericVector Binomial_American_Greeks_cpp(double initial_price = 100,
 
   NumericVector value(steps+1);
 
-  if(payoff == "call") {
+  if (payoff == "call") {
     payoff_function = &american_call;
-  } else if(payoff == "put") {
+  } else if (payoff == "put") {
     payoff_function = &american_put;
     }
 
@@ -75,7 +75,7 @@ NumericVector Binomial_American_Greeks_cpp(double initial_price = 100,
   }
 
   result("fair_value") = max(p_*value(0) + q_*value(1),
-         price(2*steps + j - steps) - exercise_price);
+         price(steps + 1) - exercise_price);
   result("delta") = (value(0) - value(1)) / (initial_price * (up - down));
   result("gamma") = (value(0) - 2*result("fair_value") + value(1)) /
     (initial_price * (up - down) * (up - down));
