@@ -4,7 +4,9 @@ using namespace Rcpp;
 // [[Rcpp::export]]
 NumericVector calc_I_2(NumericMatrix X, int steps, float dt) {
   int i;
-  NumericVector I;
+  int paths = X.nrow();
+
+  NumericVector I(paths);
   I = X(_, steps)/2 * steps * dt * steps * dt * dt;
   for(i = 1; i < steps; i++) {
     I = I + X(_, i) * (i * dt) * (i * dt) * dt;
